/*******************************************************************************
  MPLAB Harmony Project Main Source File

  Company:
    Microchip Technology Inc.
  
  File Name:
    main.c

  Summary:
    This file contains the "main" function for an MPLAB Harmony project.

  Description:
    This file contains the "main" function for an MPLAB Harmony project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state 
    machines of all MPLAB Harmony modules in the system and it calls the 
    "SYS_Tasks" function from within a system-wide "super" loop to maintain 
    their correct operation. These two functions are implemented in 
    configuration-specific files (usually "system_init.c" and "system_tasks.c")
    in a configuration-specific folder under the "src/system_config" folder 
    within this project's top-level folder.  An MPLAB Harmony project may have
    more than one configuration, each contained within it's own folder under
    the "system_config" folder.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

//Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "system/common/sys_module.h"   // SYS function prototypes
#include "tinyUART1.h"

#include "proc/p32mx220f032b.h"

#define DEBUG_MODE

#ifdef DEBUG_MODE
#define DEBUG_PRINT(x) UART1PrintString(x)
#define DEBUG_PUTNUM(x) UART1PutNum(x)
#define mBlinkLED(x) (LATBINV = _LATB_LATB15_MASK)
#else
#define DEBUG_PRINT(x)
#define DEBUG_PUTNUM(x)
#define mBlinkLED(x)
#endif

extern int MOUSE_DELTA_X;
extern int MOUSE_DELTA_Y;
extern bool MOUSE_BUTTON_L;
extern bool MOUSE_BUTTON_R;

int MSX_PC88_STATE = 0;
int MSX_PC88_DATA_X;
int MSX_PC88_DATA_Y;

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

int main ( void )
{
    /* Initialize all MPLAB Harmony modules, including application(s). */
    SYS_Initialize ( NULL );

    //---- Change Notice IRQ Setting (RB8)
    CNCONBSET = _CNCONB_ON_MASK;
    CNENBSET = _CNENB_CNIEB8_MASK;

    IPC8CLR = _IPC8_CNIP_MASK | _IPC8_CNIS_MASK;
    IPC8SET = (5 << _IPC8_CNIP_POSITION); //semi-near-highest priority

    IFS1CLR = _IFS1_CNBIF_MASK;
    IEC1SET = _IEC1_CNBIE_MASK;

    //---- Timer2 Setting
    TMR2 = 0x0200; //initial value
    PR2 = 0xffff;
//    T1CON = _T1CON_ON_MASK | (3 << _T1CON_TCKPS_POSITION); //ON,prescaler=1/256 (6.4us tick)
    T2CON = _T2CON_ON_MASK | (7 << _T2CON_TCKPS_POSITION); //ON,prescaler=1/256 (6.4us tick)

    IPC2CLR = _IPC2_T2IP_MASK | _IPC2_T2IS_MASK;
    IPC2SET = (6 << _IPC2_T2IP_POSITION); //near-highest priority

    IFS0CLR = _IFS0_T2IF_MASK;
    IEC0SET = _IEC0_T2IE_MASK;

    //---- Other Initialization
    UART1Init();

    DEBUG_PRINT( "\r\n***** Mouse Converter 88 / USB (MC88U) *****\r\n" );
    DEBUG_PRINT( "Firmware date: 2018/08/08, revision: 0.10\r\n\r\n" );

#ifdef DEBUG_MODE
    int prev_s = -1;
    int prev_x = -1;
    int prev_y = -1;
    int prev_l = 0;
    int prev_r = 0;
#endif

    while ( true )
    {
        /* Maintain state machines of all polled MPLAB Harmony modules. */
        SYS_Tasks ( );

        if (MOUSE_BUTTON_L)
            LATASET = _LATA_LATA0_MASK;
        else
            LATACLR = _LATA_LATA0_MASK;
        if (MOUSE_BUTTON_R)
            LATASET = _LATA_LATA1_MASK;
        else
            LATACLR = _LATA_LATA1_MASK;

#ifdef DEBUG_MODE
        if ((prev_x != MOUSE_DELTA_X)  || (prev_y != MOUSE_DELTA_Y) ||
            (prev_l != MOUSE_BUTTON_L) || (prev_r != MOUSE_BUTTON_R)) {
            prev_x = MOUSE_DELTA_X;
            prev_y = MOUSE_DELTA_Y;
            prev_l = MOUSE_BUTTON_L;
            prev_r = MOUSE_BUTTON_R;

            DEBUG_PRINT("X:");
            if (MOUSE_DELTA_X < 0) {
                DEBUG_PRINT("-");
                DEBUG_PUTNUM(-MOUSE_DELTA_X);
            }
            else {
                DEBUG_PRINT("+");
                DEBUG_PUTNUM(MOUSE_DELTA_X);
            }
            DEBUG_PRINT(", Y:");
            if (MOUSE_DELTA_Y < 0) {
                DEBUG_PRINT("-");
                DEBUG_PUTNUM(-MOUSE_DELTA_Y);
            }
            else {
                DEBUG_PRINT("+");
                DEBUG_PUTNUM(MOUSE_DELTA_Y);
            }
            DEBUG_PRINT(", ");
            if (MOUSE_BUTTON_L) DEBUG_PRINT("L"); else DEBUG_PRINT(".");
            if (MOUSE_BUTTON_R) DEBUG_PRINT("R"); else DEBUG_PRINT(".");
            DEBUG_PRINT("\r");
        }
#endif//DEBUG_MODE
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}

// *****************************************************************************
// *****************************************************************************
// Section: Change Notice Interrupt Handler
// *****************************************************************************
// *****************************************************************************

//___|~~~~~~~~~~~~|________|~~~~~~~~|____________|
//valid      *         *        *        *
//   |<max80>|    |<30>|   |<30>|   |<30>|
//   |<--max180-->|<max180>|<max180>|<--min300-->|[us]
//   | X[7:4]     | X[3:0] | Y[7:4] | Y[3:0]     |

#define clip8bit(x) (((x) < 0)? (((~(x) & (-128)) != 0)? -128: (x)): ((((x) & ~127) != 0)? 127: (x)))

void __ISR(_CHANGE_NOTICE_VECTOR, IPL5AUTO) _CNxInterrupt(void)
{
    if (CNSTATBbits.CNSTATB8 != 0) {
        if (PORTBbits.RB8 != 0) { //rise edge
            if ((MSX_PC88_STATE == 0) || ((TMR2 & 0xffe0) != 0x0100)) {
                MSX_PC88_DATA_X = clip8bit(-MOUSE_DELTA_X);
                MSX_PC88_DATA_Y = clip8bit(-MOUSE_DELTA_Y);
                MOUSE_DELTA_X = 0;
                MOUSE_DELTA_Y = 0;
                LATBSET = 0x000f;
                LATBCLR = (MSX_PC88_DATA_X >> 4) & 0x000f;
                if (MSX_PC88_STATE != 0)
                    mBlinkLED();
                MSX_PC88_STATE = 1;
            }
            else if (MSX_PC88_STATE == 2) {
                LATBSET = 0x000f;
                LATBCLR = (MSX_PC88_DATA_Y >> 4) & 0x000f;
                MSX_PC88_STATE = 3;
            }
            else { //error
                mBlinkLED();
                MSX_PC88_STATE = 0;
            }
        }
        else { //fall edge
            if ((TMR2 & 0xffe0) != 0x0100) { //timing error
                mBlinkLED();
                MSX_PC88_STATE = 0;
            }
            else if (MSX_PC88_STATE == 1) {
                LATBSET = 0x000f;
                LATBCLR = MSX_PC88_DATA_X & 0x000f;
                MSX_PC88_STATE = 2;
            }
            else if (MSX_PC88_STATE == 3) {
                LATBSET = 0x000f;
                LATBCLR = MSX_PC88_DATA_Y & 0x000f;
                MSX_PC88_STATE = 0;
            }
        }
        TMR2 = 0x0100; //restart
    }
    IFS1CLR = _IFS1_CNBIF_MASK;
}

// *****************************************************************************
// *****************************************************************************
// Section: Timer2 Interrupt Handler
// *****************************************************************************
// *****************************************************************************

void __ISR(_TIMER_2_VECTOR, IPL6AUTO) _Tmr2Interrupt(void)
{
    TMR2 = 0x0200;
    mBlinkLED();
    IFS0CLR = _IFS0_T2IF_MASK;
}

/*******************************************************************************
 End of File
*/

